@@ -31,10 +31,6 @@
 #include <sys/types.h>
 #include <sys/time.h>
 
-#if 0
-#include <pthread.h>
-#endif
-
 #ifdef USE_CURSES
 #include <linux/kd.h>		/* linux specific keyboard ioctls */
 #include <termios.h>
@@ -117,7 +113,7 @@ void terminal_restore_kbd_mode( void );
 
 #define END_OF_TABLE	-1
 
-struct keytable {
+static struct keytable {
   int code;
   int index;
   char pressed;
@@ -152,23 +148,8 @@ struct keytable {
 
 char keylist[KEY_WATCH_COUNT];	/* array of key states (up/down) */
 
-int kbd_thread_stop;		/* signal from main program to abort thread */
-int quit;			/* user requested to quit the program */
-int suspend;			/* user requested to suspend/resume program */
-int handled_suspend;		/* main program has been informed of suspend */
 int suspended;
 
-#if 0
-#define KBD_BUFSIZE	40
-
-int kbd_in_play_mode;
-int kbd_buffer[KBD_BUFSIZE];	/* Store keys in here when not playing the
-				   game so we can use the keyboard normally. */
-int kbd_buf_head, kbd_buf_tail;
-
-pthread_t *kbd_thread;
-#endif
-
 
 enum KbdMode curr_km;
 
@@ -181,69 +162,6 @@ int (*kbd_ptr_get_action)( struct kbd_action *ka ) = NULL;
 void (*kbd_ptr_get_action_from_keylist_helper)( void ) = NULL;
 
 
-
-#if 0
-
-/* Why is this thread here?  Because getchar() is normally blocking
- * and I used to think I couldn't change that.  Therefore, I put
- * it in a separate thread.  As it turns out, you can make getchar()
- * nonblocking, so I'm trying to get by without this thread.  We
- * shall see.
- */
-
-void *keyboard_thread( void *p )
-{
-  int i, ch;
-  
-  while(1) {
-    if( kbd_thread_stop )
-      return NULL;
-
-    ch=getchar();
-    
-    if( kbd_in_play_mode ) {
-      i=0;
-      while( keymap[i].code != END_OF_TABLE ) {
-	if( ch == keymap[i].code ) {
-	  keylist[keymap[i].index]=keymap[i].pressed;
-	  break;
-	}
-	i++;
-      }
-      
-      /* CTRL-Z: Pause game and restore normal kbd functionality */
-      if( keylist[CTRL_KEY] && ch == KEYBOARD_KEY_DOWN_Z ) {
-	suspend = 1;
-
-	terminal_restore_kbd_mode();
-	while( getchar() != '\r' )
-	  ;
-	terminal_set_raw_kbd_mode();
-	suspend = 2;
-
-	continue;
-      }
-      
-      /* CTRL-C: Quit */
-      if( keylist[CTRL_KEY] && ch == KEYBOARD_KEY_DOWN_C ) {
-	quit = 1;
-	return NULL;
-      }
-    }
-    else {
-      if( ch < 128 ) {		/* key pressed */
-	/* FIXME: use mutex. */
-	if( (kbd_buf_head+1)%KBD_BUFSIZE != kbd_buf_tail ) {
-	  kbd_buf_head = (kbd_buf_head+1)%KBD_BUFSIZE;
-	  kbd_buffer[kbd_buf_head] = ch;
-	}
-      }
-    }
-  }
-}
-#endif
-
-
 void kbd_setmode_no_op( enum KbdMode km )
 {
 }
@@ -300,6 +218,7 @@ void kbd_console_process_all_actions( void )
   int i, ch;
   while( (ch = getchar()) != EOF && ch != 0 ) {
     i=0;
+    ch = translate_raw_key(ch);
     while( keymap[i].code != END_OF_TABLE ) {
       if( ch == keymap[i].code ) {
 	keylist[keymap[i].index]=keymap[i].pressed;
@@ -316,9 +235,15 @@ int non_raw_mode_get_kbd_action( struct kbd_action *ka )
   int key;
   int retval=0;
   
+  ka->action=MOVE_ACTION;
   ka->accel = 0;
+  ka->fx = ka->fy = ka->mx = ka->my = 0;
+
+  /* Use getch() here and not getchar() so that ncurses can do the
+   * work of translating arrow keys into something reasonable.
+   */
+  while( (key = getch()) != EOF && key != 0 ) {
 
-  while( (key = getchar()) != EOF && key != 0 ) {
 #if 0
     if( suspended ) {
       suspended = 0;
@@ -395,27 +320,20 @@ void kbd_raw_curses_setmode( enum KbdMode km )
 }
 
 
-int kbd_threaded_raw_curses_getch( void )
+int kbd_raw_curses_getch( void )
 {
   return getch();
 }
 
 
-int kbd_threadless_cooked_curses_getch( void )
+int kbd_cooked_curses_getch( void )
 {
   return getch();
 }
 
 
-int kbd_threaded_raw_curses_close( void )
+int kbd_raw_curses_close( void )
 {
-#if 0
-  void *status;
-
-  kbd_thread_stop = 1;
-  pthread_join( *kbd_thread, &status );
-  free( kbd_thread );
-#endif
   terminal_restore_kbd_mode();
   terminal_restore_canonical_mode();
   terminal_close();
@@ -424,7 +342,7 @@ int kbd_threaded_raw_curses_close( void )
 }
 
 
-int kbd_threadless_cooked_curses_close( void )
+int kbd_cooked_curses_close( void )
 {
   terminal_restore_canonical_mode();
   
@@ -432,7 +350,7 @@ int kbd_threadless_cooked_curses_close( void )
 }
 
 
-/* kbd_threaded_raw_curses_emergency_close( void )
+/* kbd_raw_curses_emergency_close( void )
  * 
  * This is called from a signal handler so that we can try to set
  * the keyboard back to cooked mode but ioctl() is probably not
@@ -441,18 +359,18 @@ int kbd_threadless_cooked_curses_close( void )
  * though.
  */
 
-void kbd_threaded_raw_curses_emergency_close( void )
+void kbd_raw_curses_emergency_close( void )
 {
   terminal_restore_kbd_mode();
 }
 
 
-void kbd_threadless_cooked_curses_emergency_close( void )
+void kbd_cooked_curses_emergency_close( void )
 {
 }
 
 
-int kbd_threadless_cooked_curses_init( void )
+int kbd_cooked_curses_init( void )
 {
   if( !terminal_set_noncanonical_mode() )
     return 0;
@@ -465,10 +383,10 @@ int kbd_threadless_cooked_curses_init( void )
 #endif
   suspended = 0;
 
-  kbd_ptr_close = kbd_threadless_cooked_curses_close;
-  kbd_ptr_emergency_close = kbd_threadless_cooked_curses_emergency_close;
+  kbd_ptr_close = kbd_cooked_curses_close;
+  kbd_ptr_emergency_close = kbd_cooked_curses_emergency_close;
   kbd_ptr_playmode = kbd_setmode_no_op;
-  kbd_ptr_getch = kbd_threadless_cooked_curses_getch;
+  kbd_ptr_getch = kbd_cooked_curses_getch;
   kbd_ptr_get_action = non_raw_mode_get_kbd_action;
   kbd_ptr_get_action_from_keylist_helper = NULL;
   
@@ -476,34 +394,26 @@ int kbd_threadless_cooked_curses_init( void )
 }
 
 
-int kbd_threaded_raw_curses_init( void )
+int kbd_raw_curses_init( void )
 {
   int i;
+
+  if( !get_config_from_dumpkeys() )
+      return 0;
   
-  if( terminal_open() )
+  if( terminal_open() ) {
     if( terminal_set_noncanonical_mode() ) {
       if( terminal_set_raw_kbd_mode() ) {
 
-	/* Initialize variables used to communicate with keyboard
-	 * thread.
+	/* Initialize keyboard state variables.
 	 */
 	for( i=0; i<KEY_WATCH_COUNT; i++ )
 	  keylist[i]=0;
-	kbd_thread_stop = 0;
-	quit=0;
-	suspend=0;
-	handled_suspend=0;
 
-#if 0
-	/* Launch the keyboard thread. */
-	kbd_thread = (pthread_t *) malloc( sizeof(pthread_t) );
-	pthread_create( kbd_thread, NULL, keyboard_thread, NULL );
-#endif
-
-	kbd_ptr_close = kbd_threaded_raw_curses_close;
-	kbd_ptr_emergency_close = kbd_threaded_raw_curses_emergency_close;
+	kbd_ptr_close = kbd_raw_curses_close;
+	kbd_ptr_emergency_close = kbd_raw_curses_emergency_close;
 	kbd_ptr_playmode = kbd_raw_curses_setmode;
-	kbd_ptr_getch = kbd_threaded_raw_curses_getch;
+	kbd_ptr_getch = kbd_raw_curses_getch;
 	kbd_ptr_get_action = get_kbd_action_from_keylist;
 	kbd_ptr_get_action_from_keylist_helper = kbd_console_process_all_actions;
 	
@@ -517,6 +427,7 @@ int kbd_threaded_raw_curses_init( void )
     else {
       terminal_close();
     }
+  }
 
   return 0;
 }
@@ -689,9 +600,9 @@ int init_kbd( enum DisplayType disptype, int use_raw_mode )
 #if defined(USE_CURSES) || defined(DOUBLEWIDE_FONT_HACK)
     case DT_CURSES:
     case DT_CURSES_DOUBLEWIDE:
-      if( use_raw_mode && kbd_threaded_raw_curses_init() )
+      if( use_raw_mode && kbd_raw_curses_init() )
 	return 1;
-      return kbd_threadless_cooked_curses_init();
+      return kbd_cooked_curses_init();
       break;
 #endif
 #ifdef USE_SVGALIB
