@@ -28,8 +28,10 @@
 #include <unistd.h>
 #include <signal.h>
 #include <time.h>
+#include <string.h>
+#include <errno.h>
+#include <sys/time.h>
 #include <sys/wait.h>
-#include <pthread.h>
 
 #include "snipes.h"
 #include "player.h"
@@ -49,7 +51,7 @@
 
 /* Times are in microseconds (1e-6) */
 
-#define	DISPLAY_TIME	10000
+#define	DISPLAY_TIME	20000
 
 /* Counts are multiples of DISPLAY_TIME */
 
@@ -431,78 +433,98 @@ void display_maze( int my_player, screen_coords *sc, wall_info *w )
 /*                        CORE GAME FUNCTIONS                        */
 /* ----------------------------------------------------------------- */
 
-
-/* The following variables are used by display_count() and run_snipes().
- * display_count() acts as a clock which permits run_snipes() to execute
- * one iteration of the main loop every so many milliseconds.
+/* alarm_occurred is set to one by the SIGALRM interrupt handler.
+ * SIGALRMs are generated by the real (wall-clock time) interval
+ * timer and are normally blocked.  They are unblocked by
+ * sigsuspend() when the program is done with a display cycle.
  */
 
-pthread_mutex_t	display_mutex = PTHREAD_MUTEX_INITIALIZER;
-pthread_cond_t	display_cond  = PTHREAD_COND_INITIALIZER;
-int ready_to_display=1;
-pthread_t *count_thread;
+int alarm_occurred;
 
 
-/* display_count( void *p )
+/* sig_alrm( int signo )
  * 
- * Timer thread which controls the execution of the main loop in
- * run_snipes().
+ * Signal handler for ALRM (interval timer).
  */
 
-void *display_count( void *p )
+void sig_alrm( int signo )
 {
-  while( 1 ) {
-    usleep( DISPLAY_TIME );
-
-    pthread_mutex_lock( &display_mutex );
-    while( ready_to_display == 1 )
-      pthread_cond_wait( &display_cond, &display_mutex );
-
-    if( ready_to_display == 2 ) {
-      pthread_mutex_unlock( &display_mutex );
-      return NULL;
-    }
-
-    ready_to_display=1;
-    pthread_cond_broadcast( &display_cond );
-    pthread_mutex_unlock( &display_mutex );
-  }
+	alarm_occurred = 1;
 }
 
 
-void start_count_thread( void )
-{
-  /* Display immediately on the first run through the loop. */
-  ready_to_display=1;
+/* start_timer( void )
+ *
+ * Set sig_alrm() as the signal handler for alarms.  Block reception
+ * of SIGALRM.  Start the real interval timer to generate SIGALRM
+ * periodically.  SIGALRMs are unblocked by sigsuspend() when
+ * the program is done with a display cycle.
+ */
 
-  count_thread = (pthread_t *) malloc( sizeof(pthread_t) );
-  pthread_create( count_thread, NULL, display_count, NULL );
+void start_timer( void )
+{
+	int ret;
+	char errstr[200];
+	struct itimerval value;
+	sigset_t sigset;
+
+	alarm_occurred = 0;
+
+	if( signal( SIGALRM, sig_alrm ) == SIG_ERR ) {
+		err_sys( "can't catch SIGALRM" );
+	}
+
+	sigemptyset( &sigset );
+	sigaddset( &sigset, SIGALRM );
+	sigprocmask( SIG_BLOCK, &sigset, NULL );
+
+	value.it_interval.tv_sec  = 0;
+	value.it_interval.tv_usec = DISPLAY_TIME;
+	value.it_value.tv_sec     = 0;
+	value.it_value.tv_usec    = DISPLAY_TIME;
+	ret = setitimer( ITIMER_REAL, &value, NULL );
+
+	if( ret ) {
+		strcpy( errstr, "Unable to start interval timer: " );
+		strncat( errstr, strerror(errno), 100 );
+		err_sys( errstr );
+	}
 }
 
 
-void end_count_thread( void )
+/* wait_timer( void )
+ *
+ * Use sigsuspend() to temporarily unblock SIGALRM.  Wait until
+ * a SIGALRM occurs.
+ */
+
+void wait_timer( void )
 {
-  void *status;
+	sigset_t sigset;
 
-  pthread_mutex_lock( &display_mutex );
-  ready_to_display=2;
-  pthread_cond_broadcast( &display_cond );
-  pthread_mutex_unlock( &display_mutex );
-
-  pthread_join( *count_thread, &status );
-  free( count_thread );
+	sigemptyset(&sigset);
+	while( !alarm_occurred )
+		sigsuspend(&sigset);
+	alarm_occurred = 0;
 }
 
 
-void display_wait( void )
-{
-  pthread_mutex_lock( &display_mutex );
-  while( ready_to_display == 0 )
-    pthread_cond_wait( &display_cond, &display_mutex );
+/* stop_timer( void )
+ *
+ * Disable the real interval timer.
+ */
 
-  ready_to_display=0;
-  pthread_cond_broadcast( &display_cond );
-  pthread_mutex_unlock( &display_mutex );
+void stop_timer( void )
+{
+	struct itimerval value;
+
+	value.it_interval.tv_sec  = 0;
+	value.it_interval.tv_usec = 0;
+	value.it_value.tv_sec     = 0;
+	value.it_value.tv_usec    = 0;
+	setitimer( ITIMER_REAL, &value, NULL );
+
+	/* return value is ignored, assume it will work okay */
 }
 
 
@@ -569,7 +591,7 @@ void run_snipes( void )
   game_start_time = time(NULL);
   elapsed_time = 0;
   
-  start_count_thread();
+  start_timer();
 
   sound_init( use_sound );
 
@@ -588,7 +610,7 @@ void run_snipes( void )
 
   while( !quit )
   {
-    display_wait();
+	wait_timer();
     
     /* Recalculate maze border coordinates. */
     
@@ -627,7 +649,7 @@ void run_snipes( void )
 
   screen_cursor_on();
 
-  end_count_thread();
+  stop_timer();
 
   close_walls( &w );
 
